@@ -15,7 +15,7 @@ use crate::arg_parser::{ArgParser, GreedyState};
 use regex::Regex;
 use crate::utils::Utils;
 use crate::processor::Processor;
-use crate::auth::{AuthState, AuthType};
+use crate::auth::AuthType;
 #[cfg(feature = "csv")]
 use crate::formatter::Formatter;
 #[cfg(feature = "lipsum")]
@@ -81,6 +81,7 @@ impl BasicMacro {
             ("path".to_owned(),    BasicMacro::merge_path       as MacroType),
             ("paths".to_owned(),   BasicMacro::merge_path_vec   as MacroType),
             ("pipe".to_owned(),    BasicMacro::pipe             as MacroType),
+            ("read".to_owned(),    BasicMacro::read             as MacroType),
             ("redir".to_owned(),   BasicMacro::temp_redirect    as MacroType),
             ("regex".to_owned(),   BasicMacro::regex_sub        as MacroType),
             ("rename".to_owned(),  BasicMacro::rename_call      as MacroType),
@@ -135,20 +136,6 @@ impl BasicMacro {
         self.macros.get(name)
     }
 
-    /// Check file authority
-    fn is_granted(name:&str, auth_type: AuthType, processor: &mut Processor) -> Result<bool, RadError> {
-        match processor.get_auth_state(&auth_type) {
-            AuthState::Restricted => {
-                Err(RadError::PermissionDenied(name.to_owned(), auth_type))
-            }
-            AuthState::Warn => {
-                processor.log_warning(&format!("\"{}\" was called with \"{:?}\" permission", name, auth_type))?;
-                Ok(true)
-            }
-            AuthState::Open => Ok(true),
-        }
-    }
-
     /// Undefine a macro
     ///
     /// # Arguments
@@ -321,11 +308,53 @@ impl BasicMacro {
     ///
     /// Every macros within the file is also expanded
     ///
+    /// Include read file's content into a single string and print out.
+    /// This enables ergonomic process of macro execution. If you want file 
+    /// inclusion to happen as bufstream, use read instead.
+    ///
     /// # Usage
     ///
     /// $include(path)
     fn include(args: &str, greedy: bool, processor: &mut Processor) -> Result<Option<String>, RadError> {
-        if !Self::is_granted("include", AuthType::FIN,processor)? {
+        if !Utils::is_granted("include", AuthType::FIN,processor)? {
+            return Ok(None);
+        }
+        if let Some(args) = ArgParser::new().args_with_len(args, 1, greedy) {
+            let raw = Utils::trim(&args[0]);
+            let mut file_path = PathBuf::from(&raw);
+
+            // if current input is not stdin and file path is relative
+            // Create new file path that starts from current file path
+            if processor.current_input != "stdin" && file_path.is_relative() {
+                // It is ok get parent because any path that has a length can return parent
+                file_path = PathBuf::from(&processor.current_input).parent().unwrap().join(file_path);
+            }
+
+            if file_path.is_file() { 
+                processor.set_sandbox();
+                let chunk = processor.from_file_as_chunk(file_path)?;
+                Ok(chunk)
+            } else {
+                let formatted = format!("File path : \"{}\" doesn't exist or not a file", file_path.display());
+                Err(RadError::InvalidArgument(formatted))
+            }
+        } else {
+            Err(RadError::InvalidArgument("Include requires an argument".to_owned()))
+        }
+    }
+
+    /// Paste given file's content as bufstream
+    ///
+    /// Every macros within the file is also expanded
+    ///
+    /// Read include given file's content as form of bufstream and doesn't 
+    /// save to memory. Therefore cannot be used with macro definition.
+    ///
+    /// # Usage
+    ///
+    /// $read(path)
+    fn read(args: &str, greedy: bool, processor: &mut Processor) -> Result<Option<String>, RadError> {
+        if !Utils::is_granted("read", AuthType::FIN,processor)? {
             return Ok(None);
         }
         if let Some(args) = ArgParser::new().args_with_len(args, 1, greedy) {
@@ -386,7 +415,7 @@ impl BasicMacro {
     ///
     /// $syscmd(system command -a arguments)
     fn syscmd(args: &str, _: bool,p: &mut Processor) -> Result<Option<String>, RadError> {
-        if !Self::is_granted("syscmd", AuthType::CMD,p)? {
+        if !Utils::is_granted("syscmd", AuthType::CMD,p)? {
             return Ok(None);
         }
         if let Some(args_content) = ArgParser::new().args_with_len(args, 1, true) {
@@ -542,7 +571,7 @@ impl BasicMacro {
     ///
     /// $env(SHELL)
     fn get_env(args: &str, _: bool, p : &mut Processor) -> Result<Option<String>, RadError> {
-        if !Self::is_granted("env", AuthType::ENV,p)? {
+        if !Utils::is_granted("env", AuthType::ENV,p)? {
             return Ok(None);
         }
         if let Ok(out) = std::env::var(args) {
@@ -789,7 +818,7 @@ impl BasicMacro {
     ///
     /// $tempout(Content)
     fn temp_out(args: &str, greedy: bool, p: &mut Processor) -> Result<Option<String>, RadError> {
-        if !Self::is_granted("tempout", AuthType::FOUT,p)? {
+        if !Utils::is_granted("tempout", AuthType::FOUT,p)? {
             return Ok(None);
         }
 
@@ -808,7 +837,7 @@ impl BasicMacro {
     ///
     /// $fileout(true,file_name,Content)
     fn file_out(args: &str, greedy: bool, p: &mut Processor) -> Result<Option<String>, RadError> {
-        if !Self::is_granted("fileout", AuthType::FOUT,p)? {
+        if !Utils::is_granted("fileout", AuthType::FOUT,p)? {
             return Ok(None);
         }
         if let Some(args) = ArgParser::new().args_with_len(args, 3, greedy) {
@@ -848,17 +877,19 @@ impl BasicMacro {
 
     /// Include but for temporary file
     ///
+    /// This reads file's content into memory. Use read macro if streamed write is needed.
+    ///
     /// # Usage
     ///
     /// $tempin()
     fn temp_include(_: &str, _: bool, processor: &mut Processor) -> Result<Option<String>, RadError> {
-        if !Self::is_granted("tempin", AuthType::FIN,processor)? {
+        if !Utils::is_granted("tempin", AuthType::FIN,processor)? {
             return Ok(None);
         }
         let file = processor.get_temp_path().to_owned();
         processor.set_sandbox();
-        processor.from_file(&file)?;
-        Ok(None)
+        let chunk = processor.from_file_as_chunk(&file)?;
+        Ok(chunk)
     }
 
     /// Redirect all text into temporary file
@@ -870,7 +901,7 @@ impl BasicMacro {
     /// $redir(true) 
     /// $redir(false) 
     fn temp_redirect(args: &str, _: bool, p: &mut Processor) -> Result<Option<String>, RadError> {
-        if !Self::is_granted("redir", AuthType::FOUT,p)? {
+        if !Utils::is_granted("redir", AuthType::FOUT,p)? {
             return Ok(None);
         }
         if let Some(args) = ArgParser::new().args_with_len(args, 1, false) {
@@ -892,7 +923,7 @@ impl BasicMacro {
     ///
     /// $tempto(file_name)
     fn set_temp_target(args: &str, greedy: bool, processor: &mut Processor) -> Result<Option<String>, RadError> {
-        if !Self::is_granted("tempto", AuthType::FOUT,processor)? {
+        if !Utils::is_granted("tempto", AuthType::FOUT,processor)? {
             return Ok(None);
         }
         if let Some(args) = ArgParser::new().args_with_len(args, 1, greedy) {
